pub unsafe fn at_module_exit(cb: extern "C" fn())Expand description
Registers a raw function to be called at library (libc calls this a DSO or “dynamic shared object”) exit time.
Corresponds to __cxa_atexit in C, though the exit function argument is
not available.
Safety
Rust does not provide any safety guarantees about life-before-main or life-after-main. Ordering of destructors is not guaranteed, nor that a destructor will be called at all.